@@ -12,10 +12,13 @@ from __future__ import unicode_literals
 import re
 import rsa
 import requests
+from datetime import datetime
+
 from thrift.transport import TTransport
 from thrift.transport import TSocket
 from thrift.transport import THttpClient
 from thrift.protocol import TCompactProtocol
+
 import sys
 reload(sys)
 sys.setdefaultencoding("utf-8")
@@ -32,11 +35,51 @@ __all__ = ['LineClient']
 
 EMAIL_REGEX = re.compile(r"[^@]+@[^@]+\.[^@]+")
 
+class LineMessage:
+    """LineMessage wrapper"""
+
+    TEXT      = 0
+    IMAGE     = 1
+    VIDEO     = 2
+    AUDIO     = 3
+    HTML      = 4
+    PDF       = 5
+    CALL      = 6
+    STICKER   = 7
+    PRESENCE  = 8
+    GIFT      = 9
+    GROUBOARD = 10
+    APPLINK   = 11
+
+    def __init__(self, client, message):
+        self._client = client
+        self.type = message.type
+        self.text = message.text
+        self.id   = message.id
+
+        self.sender   = client.getContactFromId(message._from)
+        self.receiver = client.getContactFromId(message.to)
+
+        self.sendTime = datetime.fromtimestamp(message.createdTime/1000)
+        self.contentPreview = message.contentPreview
+
+    def __repr__(self):
+        return 'LineMessage (type=%s, sender=%s, receiver=%s) "%s"' % (
+                    self.type,
+                    self.sender,
+                    self.receiver,
+                    self.text
+                )
+
 class LineGroup(object):
-    """Group object for LINE"""
+    """LineGroup wrapper
+    
+    :param client: LineClient instance
+    :param group: Group instace
+    """
     def __init__(self, client, group):
         self._client  = client
-        self._group = group
+        self._group   = group
 
         self.id      = group.id
         self.name    = group.name
@@ -46,44 +89,144 @@ class LineGroup(object):
         for member in group.members:
             self.members.append(LineContact(client, member))
 
+        self.invitee = []
+        if group.invitee:
+            for member in group.invitee:
+                self.invitee.append(LineContact(client, member))
+
     def sendMessage(self, text):
         try:
-            self._client._sendMessage(id=self.id, text=text)
+            message = CurveThrift.Message(to=self.id, text=text)
+            self._client._sendMessage(message)
+
             return True
         except Exception as e:
             raise e
 
     def getRecentMessages(self, count=1):
-        return self._client._getRecentMessages(self._group, count)
+        """Get recent messages"""
+        try:
+            messages = self._client._getRecentMessages(self.messageBox.id, count)
+            return messages
+        except:
+            messageBoxWrapUp = self._client._getMessageBoxCompactWrapUp(self.id)
+            self.messageBox = messageBoxWrapUp.messageBox
+
+            messages = self._client._getRecentMessages(self.messageBox.id, count)
+            return messages
 
     def __repr__(self):
+        """Name of Group and number of group members"""
         return '<LineGroup %s #%s>' % (self.name, len(self.members))
 
     def __lt__(self, other):
         return self.id < other.id
 
 class LineContact(object):
-    """Contact object for LINE"""
+    """LineContact wrapper
+    
+    :param client: LineClient instance
+    :param contact: Conatct instace
+    """
     def __init__(self, client, contact):
         self._client  = client
         self._contact = contact
 
-        self.mid           = contact.mid
+        self.id           = contact.mid
         self.displayName   = contact.displayName
         self.statusMessage = contact.statusMessage
 
+    def getRecentMessages(self, count=1):
+        try:
+            messages = self._client._getRecentMessages(self.messageBox.id, count)
+            return messages
+        except:
+            messageBoxWrapUp = self._client._getMessageBoxCompactWrapUp(self.id)
+            self.messageBox = messageBoxWrapUp.messageBox
+
+            messages = self._client._getRecentMessages(self.messageBox.id, count)
+            return messages
+
+    def sendImage(self, path):
+        try:
+            img = open(path, 'r')
+
+            message = CurveThrift.Message(to=self.id, text=text)
+            message.contentType = CurveThrift.ContentType.IMAGE
+            message.contentPreview = img.read().encode('utf-8')
+
+            self.raise_error("not implemented yet")
+
+            url = None
+
+            message.contentMetadata = {
+                'PREVIEW_URL': url,
+                'DOWNLOAD_URL': url,
+                'PUBLIC': True,
+            }
+
+            self._client._sendMessage(message)
+
+            return True
+        except Exception as e:
+            raise e
+
+    def sendImageWithURL(self, url):
+        try:
+            response = requests.get(url, stream=True)
+
+            message = CurveThrift.Message(to=self.id, text=None)
+            message.contentType = CurveThrift.ContentType.IMAGE
+            message.contentPreview = response.raw.read()
+            #message.contentPreview = url.encode('utf-8')
+
+            message.contentMetadata = {
+                'PREVIEW_URL': url,
+                'DOWNLOAD_URL': url,
+                'PUBLIC': "True",
+            }
+
+            self._client._sendMessage(message, seq=1)
+
+            return True
+        except Exception as e:
+            raise e
+
+    def sendSticker(self, url):
+        try:
+            response = requests.get(url, stream=True)
+
+            message = CurveThrift.Message(to=self.id, text=None)
+            message.contentType = CurveThrift.ContentType.IMAGE
+            message.contentPreview = response.raw.read()
+            #message.contentPreview = url.encode('utf-8')
+
+            message.contentMetadata = {
+                'PREVIEW_URL': url,
+                'DOWNLOAD_URL': url,
+                'PUBLIC': "True",
+            }
+
+            self._client._sendMessage(message, seq=1)
+
+            return True
+        except Exception as e:
+            raise e
+
     def sendMessage(self, text):
         try:
-            self._client._sendMessage(id=self.mid, text=text)
+            message = CurveThrift.Message(to=self.id, text=text)
+            self._client._sendMessage(message)
+
             return True
         except Exception as e:
             raise e
 
     def __repr__(self):
-        return '<LineContact %s (%s)>' % (self.displayName, self.mid)
+        return '<LineContact %s (%s)>' % (self.displayName, self.id)
 
     def __lt__(self, other):
-        return self.mid < other.mid
+        return self.id < other.id
 
 class LineClient(object):
     """This class proviede you a way to communicate with LINE server.
@@ -105,6 +248,7 @@ class LineClient(object):
     version     = "3.7.0"
     com_name    = ""
 
+    revision = 0
     profile  = None
     contacts = []
     rooms    = []
@@ -159,10 +303,15 @@ class LineClient(object):
             self.login()
             self.ready()
 
+        self.revision = self._getLastOpRevision()
         self.refreshContacts()
         self.refreshGroups()
 
     def ready(self):
+        """
+        After login, make `client` and `client_in` instance
+        to communicate with LINE server
+        """
         self.transport    = THttpClient.THttpClient(self.LINE_HTTP_URL)
         self.transport_in = THttpClient.THttpClient(self.LINE_HTTP_IN_URL)
 
@@ -254,7 +403,15 @@ class LineClient(object):
             self.contacts.append(self.profile)
             self.contacts.sort()
 
+    def getContactFromId(self, id):
+        for contact in self.contacts:
+            if contact.id == id:
+                return contact
+
+        return None
+
     def raise_error(self, msg):
+        """Fix a error format"""
         raise Exception("Error: %s" % msg)
 
     def get_json(self, url):
@@ -338,16 +495,59 @@ class LineClient(object):
             return self.client.getGroups(ids)
 
     def _getRecentMessages(self, id, count=1):
+        """Get recent messages from `id`"""
         if self.check_auth():
             return self.client.getRecentMessages(id, count)
 
-    def _sendMessage(self, id, text, seq=0):
-        if self.check_auth():
-            message = CurveThrift.Message(to=id,text=text)
+    def _sendMessage(self, message, seq=0):
+        """Send a message to `id`. `id` could be contact id or group id
 
+        :param id: `contact` id or `group` id
+        :param message: `message` instance
+        """
+        if self.check_auth():
             return self.client.sendMessage(seq, message)
 
+    def longPoll(self):
+        """Check is there any operations from LINE server"""
+        OT = CurveClient.OperationType
+
+        try:
+            operations = self.client_in.fetchOperations(self.revision, 50)
+        except EOFError:
+            # long-poll timeout
+            return
+        except TalkException as e:
+            if e.code == 8:
+                msg = "user logged in to another machine."
+                raise_error(msg)
+            else:
+                return
+
+        for operation in operations:
+            if operation.type == OT.END_OF_OPERATION:
+                pass
+            elif operation.type == OT.SEND_MESSAGE:
+                print operation.message.text
+                pass
+            elif operation.type == OT.RECEIVE_MESSAGE:
+                pass
+
+            self.revision = max(operation.revision, self.revision)
+
     def _getLastOpRevision(self):
         if self.check_auth():
             return self.client.getLastOpRevision()
 
+    def _fetchOperations(self, revision, count=50):
+        if self.check_auth():
+            return self.client.fetchOperations(revision, count)
+
+    def _getMessageBoxCompactWrapUp(self, id):
+        if self.check_auth():
+            try:
+                return self.client.getMessageBoxCompactWrapUp(id)
+            except Exception as e:
+                msg = e
+                raise_error(msg)
+
